import argparse
import importlib.util
import re
import sys
import shlex
import os
from multiprocessing import cpu_count


def parse_options(user_options=None):
    parser = argparse.ArgumentParser(prog='iTesting', description='iTesting framework demo')
    parser.add_argument('-env', action='store', default=None, dest='default_env', metavar='target environment',
                        help='Specify test environment')
    parser.add_argument('-t', action='store', default=None, dest='test_targets',
                        metavar='target run path/file',
                        help='Specify run path/file')
    parser.add_argument("-i", action="store", default=None, dest="include_tags_any_match", metavar="user provided tags, \
        string only, separate by comma without an spacing among all tags. if any user provided \
        tags are defined in test class, the test class will be considered to run.",
                 help="Select test cases to run by tags, separated by comma, no blank space among tag values.")
    # 添加并发数目，默认为可用的CPU个数
    parser.add_argument("-n", action="store", type=int, default=cpu_count(), dest="test_thread_number",
                        metavar="int number", help="Specify the number of testing thread run in parallel, \
                        default are the cpu number.")
    parser.add_argument('-r', action='store', dest="report_file", default=None,
                        metavar='store report base file',
                        help='Specify report base file')

    if not user_options:
        args = sys.argv[1:]
    else:
        args = shlex.split(user_options)

    options, un_known = parser.parse_known_args(args)

    def split(option_value):
        return None if option_value is None else re.split(r'[;,\s]\s*',  option_value)

    options.include_tags_any_match = split(options.include_tags_any_match)

    # 指定并发数目，默认为可用CPU个数
    options.test_thread_number = options.test_thread_number

    if options.test_targets:
        if not os.path.isdir(options.test_targets) and not os.path.isfile(options.test_targets):
            parser.error("Test targets must either be a folder path or a file path, it either be absolute path \
                         or relative path, if it is relative , it must relative to tests folder under your root folder")
        if not os.path.isabs(options.test_targets):
            options.test_targets = os.path.abspath(options.test_targets)
        else:
            options.test_targets = options.test_targets

    if options.default_env:
        try:
            module_package = '.' + os.sep + 'settings' + os.sep + 'test_env'
            module_name = module_package + os.sep + '{}_env'.format(options.default_env)
            module_file = '{}.py'.format(module_name)
            module_spec = importlib.util.spec_from_file_location(module_name, module_file)
            env = importlib.util.module_from_spec(module_spec)
            module_spec.loader.exec_module(env)
            options.config = {item: getattr(env, item, None) for item in dir(env) if not item.startswith("__")}
        except ImportError:
            raise ImportError('Module: {} can not imported'.format(module_name))

    return options



import re


class TestFilter:
    def __init__(self, test_suites):
        self.suites = test_suites
        self.exclude_suites = None

    # 标签筛选策略，只要包括，即运行。
    def filter_tags_in_any(self, user_option_tags):
        """只要包括某个标签，即运行"""

        included_cls = []
        remain_cases = []
        excluded_cases = []

        for i in self.suites:
            tags_in_class = i[0]
            tags = []

            def recursion(raw_tag):
                if raw_tag:
                    if isinstance(raw_tag, (list, tuple)):
                        for item in raw_tag:
                            if isinstance(item, (list, tuple)):
                                recursion(item)
                            else:
                                tags.append(item)
                    else:
                        return re.split(r'[;,\s]\s*', raw_tag)
                return tags

            after_parse = recursion(tags_in_class)

            if any(map(lambda x: True if x in after_parse else False, user_option_tags)):
                included_cls.append(i[0])

        for s in self.suites:
            if s[0] in set(included_cls):
                remain_cases.append(s)
            else:
                excluded_cases.append(s)

        self.suites = remain_cases
        self.exclude_suites = excluded_cases

    # 根据标签，依据不同的标签筛选策略进行测试用例筛选
    def tag_filter_run(self, in_any_tags):
        if in_any_tags:
            self.filter_tags_in_any(in_any_tags)

        return self.suites, self.exclude_suites



class RunTimeTooLong(Exception):
    def __init__(self, case_name, run_time):
        self.name = case_name
        self.value = run_time

    def __str__(self):
        return "Run Time Too Long Error: %s run time - %s s" % (self.name, self.value)



from setuptools import setup, find_packages

setup(
    name='ztDemo',
    version='0.0.24',
    description='a common test framework support for both UI and API test with run in parallel ability.',
    author='kevin.cai',
    author_email='不告诉你@outlook.com',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(include=['ztDemo', 'ztDemo.*']),
    install_requires=[
        'requests',
        'selenium',
        'concurrent_log_handler',
        'PyYAML',
        'openpyxl',
        'PyMySQL',
        'assertpy',
        'xlrd'
    ],
    license='MIT',
    url='https://www.testertalk.com',
    entry_points={
        'console_scripts': [
            'ztDemo = ztDemo.main:main'
        ]
    }
)



# -*- coding: utf-8 -*-


import importlib
import os
from importlib import util

import inspect
import sys

from ztDemo.common import mk_test_name


class DiscoverTestCases:
    def __init__(self, target_file_or_path=None):
        self.target_file_or_path = target_file_or_path

    def find_test_module(self):
        """根据指定文件（夹）查找测试用模块并且导入它们"""
        mod_ref = []
        file_lists = []
        # 判断是否文件夹， 如是，则递归解析出为一个个模块，并放入列表。
        if os.path.isdir(self.target_file_or_path):
            def recursive_file_parser(file_path):
                files = os.listdir(file_path)
                for f in files:
                    f_p = os.path.join(file_path, f)
                    if os.path.isdir(f_p):
                        recursive_file_parser(f_p)
                    else:
                        if f_p.endswith('.py') and not f_p.endswith('__init__.py'):
                            file_lists.append(os.path.join(file_path, f_p))
                return file_lists

            all_files = recursive_file_parser(self.target_file_or_path)

            module_name_list = []
            module_file_paths = []
            for item in all_files:
                # 根据文件路径查找模块
                module_name_list.append(inspect.getmodulename(item))
                module_file_paths.append(item)
        elif os.path.isfile(self.target_file_or_path) and not (self.target_file_or_path.endswith('__init__.py')):
            module_name_list = [inspect.getmodulename(self.target_file_or_path)]
            module_file_paths = [self.target_file_or_path]

        for module_name, module_file_path in zip(module_name_list, module_file_paths):
            try:
                module_spec = importlib.util.spec_from_file_location(
                    module_name, module_file_path)
                module = importlib.util.module_from_spec(module_spec)

                module_spec.loader.exec_module(module)
                sys.modules[module_name] = module
                mod_ref.append(module)
            except ImportError:
                raise ImportError('Module: {} can not imported'.format(self.target_file_or_path))

        return mod_ref

    def find_tests(self, mod_ref):
        """遍历上一步查找到的测试模块，过滤出符合条件的测试用例"""
        test_cases = []
        for module in mod_ref:
            cls_members = inspect.getmembers(module, inspect.isfunction(module))
            for cls in cls_members:
                cls_name, cls_code_object = cls
                for func_name in dir(cls_code_object):
                    tests_suspect = getattr(cls_code_object, func_name)
                    if getattr(tests_suspect, "__test_case_type__", None) == "__TestCase__":
                        if getattr(tests_suspect, "__test_case_enabled__", None):
                            # 获取测试函数的__test_tag__的值
                            tag_filter = getattr(tests_suspect, "__test_tag__", None)
                            # 获取测试类中的所有方法，并且查看是否有"__data_Provider__"属性。
                            if hasattr(tests_suspect, "__data_Provider__"):
                                for i, v in enumerate(getattr(tests_suspect, "__data_Provider__")):
                                    # 根据测试数据的组数调用mk_test_name生成新的测试用例名。
                                    new_test_name = mk_test_name(tests_suspect.__name__, getattr(v, "__name__", v), i)
                                    # 将测试函数的Tag，测试类，新生成的测试用例名，测试数据组成一条测试用例供测试框架后续调用。
                                    test_cases.append((tag_filter, cls_code_object, new_test_name, tests_suspect, v))
                            else:
                                # 当没有"__data_Provider__"属性时，直接返回原测试用例
                                test_cases.append((tag_filter, cls_code_object, func_name, tests_suspect, None))

        return test_cases
